/************************************************************************************************
 *
 *  File:           buttons.c
 *
 *  Compilers:      HT-PICC STD
 *                  HT-PICC18 STD
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:
 *
 ************************************************************************************************/




//******************************************************************************
//
//      Includes
//
//******************************************************************************

#include "hw_profile.h"
#include "buttons.h"


//******************************************************************************
//
//      Constants
//
//******************************************************************************

#if (BUTTONS_HOLD_TIME / BUTTONS_TASK_INTERVAL) >= 64
#error "BUTTONS: Relation HOLD_TIME / TASK_INTERVAL must be less than 64!"
#endif


//******************************************************************************
//
//      Variables
//
//******************************************************************************

volatile TBUTTONS m_Buttons;

//******************************************************************************
//
//      Function prototypes
//
//******************************************************************************

void ProcessButtons(void);


//******************************************************************************
//
//      Functions
//
//******************************************************************************

/********************************************************************************
 *
 *  TASK: Task_Buttons
 *
 *------------------------------------------------------------------------------
 *
 *  description:    Working with u pto 8 buttons. (Debounce, hold, events)
 *
 ********************************************************************************/

void ProcessButtons(void)
{
  static unsigned char s_ucPort; // Previous buttons state
  unsigned char ucTemp;

  ucTemp = 0x00;

#ifdef pin_BUTTON0
  if (!pin_BUTTON0) ucTemp |= 0x01;
#endif
#ifdef pin_BUTTON1
  if (!pin_BUTTON1) ucTemp |= 0x02;
#endif
#ifdef pin_BUTTON2
  if (!pin_BUTTON2) ucTemp |= 0x04;
#endif
#ifdef pin_BUTTON3
  if (!pin_BUTTON3) ucTemp |= 0x08;
#endif
#ifdef pin_BUTTON4
  if (!pin_BUTTON4) ucTemp |= 0x10;
#endif
#ifdef pin_BUTTON5
  if (!pin_BUTTON5) ucTemp |= 0x20;
#endif
#ifdef pin_BUTTON6
  if (!pin_BUTTON6) ucTemp |= 0x40;
#endif
#ifdef pin_BUTTON7
  if (!pin_BUTTON7) ucTemp |= 0x80;
#endif

  ucTemp ^= s_ucPort; // Changes sinse last execution
  s_ucPort ^= ucTemp; // Current pins state

  if (ucTemp) { // Debounce
    m_Buttons.ucJitter = 0;
  } else {

    //------------------------------------------------------------------------------

    m_Buttons.ucJitter++;
    ucTemp = s_ucPort ^ m_Buttons.ucPressed; // Pins changing mask

    if (!m_Buttons.ucJitter && ucTemp) { // State was changed and debounced
      //------------------------------------------------------------------------------
      //  Check for released buttons
      //------------------------------------------------------------------------------

      if (m_Buttons.ucPressed & ucTemp) { // We have released buttons
        //------------------------------------------------------------------------------
#ifdef BUTTONS_ENABLE_JUST_RELEASED
        m_Buttons.ucJustReleased |= ucTemp;
#endif
        //------------------------------------------------------------------------------
#ifdef BUTTONS_ENABLE_CHANGED
        m_Buttons.ucChanged |= ucTemp;
#endif
        //------------------------------------------------------------------------------
#ifdef  BUTTONS_ENABLE_SHORT_HOLD
        m_Buttons.ucShort |= ucTemp;
        m_Buttons.ucShort &= ~m_Buttons.ucHold;
        m_Buttons.ucHold &= ~ucTemp;
#endif
      } // if released

      //------------------------------------------------------------------------------
      //  Check for pressed buttons
      //------------------------------------------------------------------------------

      if ((m_Buttons.ucPressed ^ ucTemp) & ucTemp) { // We have pressed buttons
        //------------------------------------------------------------------------------
#ifdef BUTTONS_ENABLE_CHANGED
        m_Buttons.ucChanged |= ucTemp;
#endif
        //------------------------------------------------------------------------------
#ifdef BUTTONS_ENABLE_JUST_PRESSED
        m_Buttons.ucJustPressed |= ucTemp;
#endif
        //------------------------------------------------------------------------------

#ifdef  BUTTONS_ENABLE_SHORT_HOLD
        m_Buttons.ucShort &= ~ucTemp;
#endif

      } // if pressed

      m_Buttons.ucPressed = s_ucPort;

#ifdef  BUTTONS_ENABLE_SHORT_HOLD
      m_Buttons.ucCounter = 0;
#endif
    } // if changed and debounced

#ifdef  BUTTONS_ENABLE_SHORT_HOLD
    m_Buttons.ucCounter++;
    //------------------------------------------------------------------------------
    //  Check for holding
    //------------------------------------------------------------------------------

    if (m_Buttons.ucCounter == (BUTTONS_HOLD_TIME / BUTTONS_TASK_INTERVAL)) {
#ifdef BUTTONS_ENABLE_CHANGED
      m_Buttons.ucChanged |= m_Buttons.ucHold ^ m_Buttons.ucPressed;
#endif

      m_Buttons.ucHold = m_Buttons.ucPressed;
    }
#endif  // BUTTONS_ENABLE_SHORT_HOLD
  } // if (ucTemp)
}

//******************************************************************************
//
//      THE END (buttons.c)
//
//******************************************************************************

