#include <stdint.h>
#include <string.h>
#include "xprintf.h"

#include "crc8.h"
#include "ow_rom.h"
#include "ds18b20.h"

typedef struct {

    union {
        uint8_t spad[9];

        struct {
            uint8_t LSB;
            uint8_t MSB;
            uint8_t T_H;
            uint8_t T_L;
            uint8_t conf;
            uint8_t r1; /* 0xFF */
            uint8_t r2;
            uint8_t r3; /* 0x10 */
            uint8_t CRC;
        };
    };
} tSPAD;

uint8_t ndevs = 0;
uint8_t ROMs[MAX_TERMOS][8];

static DS18B20_RESULT convert_scratch(void* asPAD, float *term)
{
    tSPAD* sPAD = (tSPAD *)asPAD;
    unsigned char negative_tmp;
    unsigned int absolute_tmp;

    // two's complement temperature conversion
    if (sPAD->MSB & 0x8) {
        negative_tmp = 1;
    } else {
        negative_tmp = 0;
    }
    absolute_tmp = (sPAD->MSB << 8) | sPAD->LSB;
    if (negative_tmp) {
        absolute_tmp = ~absolute_tmp + 1; // 1s complement
    }
    *term = (float) absolute_tmp * .0625;
    if (negative_tmp) {
        *term *= -1;
    }
    return DS18B20_OK;
}

DS18B20_RESULT start_convert_single(void)
{
    if (!OWReset()) {
        return DS18B20_NOTFOUND;
    }
    OWWriteByte(SKIP_ROM);
    OWWriteByte(CONVERT_T);

    return DS18B20_OK;
}

DS18B20_RESULT start_convert(uint8_t dev)
{
    if (!OWReset()) {
        return DS18B20_NOTFOUND;
    }
    memmove(ROM_NO, ROMs[dev], 8);
    OWMatch();
    OWWriteByte(CONVERT_T);

    return DS18B20_OK;
}

DS18B20_RESULT read_temperature_single(float *temp_c)
{
    uint8_t get[10];
    int k;

    if (!OWReset()) {
        *temp_c = 99;
        return DS18B20_NOTFOUND;
    }
    OWWriteByte(SKIP_ROM);
    OWWriteByte(READ_SCRATCHPAD); // Read Scratch Pad
    for (k = 0; k < 9; k++) {
        get[k] = OWReadByte();
//        xprintf("%02X ", get[k]);
    }
    if (crc8(get, 9) != get[0]) {
//        return DS18B20_CRCERR;
    }
    convert_scratch(get, temp_c);

    return DS18B20_OK;
}

DS18B20_RESULT read_temperature(uint8_t dev, float *temp_c)
{
    unsigned char get[10];
    int k;

    if (!OWReset()) {
        *temp_c = 99;
        return DS18B20_NOTFOUND;
    }

    memmove(ROM_NO, ROMs[dev], 8);
    OWMatch();
    OWWriteByte(READ_SCRATCHPAD); // Read Scratch Pad
    for (k = 0; k < 9; k++) {
        get[k] = OWReadByte();
    }
    if (crc8(get, 9) != get[0]) {
        //return DS18B20_CRCERR;
    }
    convert_scratch(get, temp_c);

    return DS18B20_OK;
}

DS18B20_RESULT setTermoROM(uint8_t *ROM)
{
    if ((ndevs + 1) < MAX_TERMOS) {
        memmove(ROMs[ndevs++], ROM, 8);
        return DS18B20_OK;
    } else {
        return DS18B20_TOOMANYDEVS;
    }
}

inline uint8_t getTermoCount(void)
{
    return ndevs;
}
