/*****************************************************************************
 *
 * LCD
 *
 *****************************************************************************/

#include "lcd_i2c.h"
#include "lcd_text.h"
#include "i2c_hw.h"

volatile LCDFLAGS LCDFlags;
volatile LCDREFRESHFLAGS LCDRefreshFlags;

unsigned char LCD1Contrast_low = 0b10111100;
unsigned char LCD1Contrast_med = 0b11100000;
unsigned char LCD1Contrast_high = 0b11111100;

/*****************************************************************************
 *
 * TIC32 I2C1
 *
 *****************************************************************************/

void I2C1Command(unsigned char LCDCommand)
{
    I2C1Send(LCDCMD);
    I2C1Send(LCDCommand);
};

void LCD1Clear(void)
{
    unsigned int c = 0;
    unsigned char *Dest;
    Dest = (unsigned char*) LCD1Buffer;

    if (_LCDInv) {
        while (c++ != 512)
            *(Dest++) = 0XFF;
    } else {
        while (c++ != 512)
            *(Dest++) = 0;
    };


};

void LCD1ClearXY(unsigned char X, unsigned char Y, unsigned char dX, unsigned char dY)
{
    unsigned char X1, Y1, X2, Y2;
    unsigned char *Dest;

    X1 = X;
    Y1 = Y;
    X2 = X + dX;
    Y2 = Y + dY;


    if (X1 < 65) {
        if ((Y1 / 2) == 0) {
            _LCD1Refresh0 = 1;
        };
        if ((Y2 / 2) == 1) {
            _LCD1Refresh2 = 1;
        };
    };
    if (X2 > 64) {
        if ((Y1 / 2) == 0) {
            _LCD1Refresh1 = 1;
        };
        if ((Y2 / 2) == 1) {
            _LCD1Refresh3 = 1;
        };
    };


    Y2 += 1;
    while (Y1 != Y2) {
        Dest = &(LCD1Buffer[Y1++][X1]);
        X = X1;
        while (X++ != X2) {
            if (_LCDInv) {
                *(Dest++) = 0XFF;
            } else {
                *(Dest++) = 0;
            };
        };
    }
};

void LCD1Refresh(void)
{
    unsigned char CurX = 0, CurY = 0;
    unsigned char *Addr;

    I2C1Restart(LCDADDR);
    I2C1Command(1);
    I2C1Command(0b01000000);
    I2C1Command(0b10000000);

    I2C1Restart(LCDADDR);
    I2C1Send(0b01000000);

    Addr = (unsigned char*) LCD1Buffer;

    do {
        I2C1Send(*(Addr++));
        CurX += 1;
        if (CurX == 128) {
            CurX = 0;
            CurY += 1;
            I2C1Restart(LCDADDR);
            I2C1Command(1);
            I2C1Command(0b01000000 | CurY);
            I2C1Command(0b10000000);
            I2C1Restart(LCDADDR);
            I2C1Send(0b01000000);

        };
    } while (CurY < 4);

};

void LCD1PartialRefresh(void)
{
    unsigned char c = 0;
    unsigned char *Addr;


    if (_LCD1Refresh0) {

        Addr = (unsigned char*) LCD1Buffer;

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000000); // Y = 0
        I2C1Command(0b10000000); // X = 0

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));

        Addr = &(LCD1Buffer[1][0]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000001); // Y = 1
        I2C1Command(0b10000000); // X = 0

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));
    };



    if (_LCD1Refresh1) {

        Addr = &(LCD1Buffer[0][64]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000000); // Y = 0
        I2C1Command(0b11000000); // X = 64

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));

        Addr = &(LCD1Buffer[1][64]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000001); // Y = 1
        I2C1Command(0b11000000); // X = 64

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));
    };

    if (_LCD1Refresh2) {

        Addr = &(LCD1Buffer[2][0]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000010); // Y = 2
        I2C1Command(0b10000000); // X = 0

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));

        Addr = &(LCD1Buffer[3][0]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000011); // Y = 3
        I2C1Command(0b10000000); // X = 0

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));
    };



    if (_LCD1Refresh3) {

        Addr = &(LCD1Buffer[2][64]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000010); // Y = 2
        I2C1Command(0b11000000); // X = 64

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));

        Addr = &(LCD1Buffer[3][64]);

        I2C1Restart(LCDADDR);
        I2C1Command(1);
        I2C1Command(0b01000011); // Y = 3
        I2C1Command(0b11000000); // X = 64

        I2C1Restart(LCDADDR);
        I2C1Send(0b01000000);
        c = 0;
        while (c++ != 64) I2C1Send(*(Addr++));
    };
    _LCD1RefreshFlags = 0;


};

void LCD1SetContrast(unsigned char Value)
{
    I2C1Restart(LCDADDR);

    I2C1Command(1);
    I2C1Command(0b00001010);
    I2C1Command(0b10000000 | Value);
};

void LCD1Init(unsigned char contrast)
{
    LCD_RESET_CONFIG();
    LCD_RESET_ON();
    __delay_ms(10);
    LCD_RESET_OFF();

    I2C1Start(LCDADDR);

    I2C1Command(1);
    I2C1Command(0b00100000);
    I2C1Command(0b00001001);
    I2C1Command(0b00001100);
    I2C1Command(0b00000101);
    I2C1Command(0b00010100);

    I2C1Command(1);
    I2C1Command(0b00001010); // Contrast
    I2C1Command(0b00001010);
    I2C1Command(0b00100000);
    I2C1Command(0b00000101); // !!!!!
    LCD1SetContrast(contrast);

    LCD1Clear();
    LCD1Refresh();
    LCD1CurX = 0;
    LCD1CurY = 0;
    LCD1AutoRefresh = 0;

    _LCD1RefreshFlags = 0b1111;
    
};

/*********************************************************************
 * EOF
 ********************************************************************/
