#include "hw_profile.h"
#include "ow_phy.h"
#include <libpic30.h>


void ow_init(void) {
}

//////////////////////////////////////////////////////////////////////////////
// ow_reset()
// Performs a reset on the one-wire bus and
// returns the presence detect. Reset is 480us. Presence checked
// another 70us later.
//
unsigned char ow_reset(void) {
  unsigned char presence;
  DQwrite = 0; //pull DQ line low
  __delay_us(480);
  DQwrite = 1; // allow line to return high
  __delay_us(70);	// wait for presence
  presence = DQread; // get presence signal
  __delay_us(410);
  return(!presence); // presence signal returned
} // 0=presence, 1 = no part

//////////////////////////////////////////////////////////////////////////////
// read_bit()
// Reads a bit from the one-wire bus.
//
unsigned char ow_read_bit(void) {
  unsigned char rd_val;
  DQwrite = 0; // pull DQ low to start timeslot
  __delay_us(1);
  DQwrite = 1; // then return high
  __delay_us(10) ;
  rd_val = DQread; // return value of DQ line
  __delay_us(50);
  return rd_val;
}

//////////////////////////////////////////////////////////////////////////////
// write_bit()
// Writes a bit to the one-wire bus, passed in bitval.
//
void ow_write_bit(unsigned char bitval) {
  DQwrite = 0; // pull DQ low to start timeslot
  __delay_us(1);
  if (bitval == 1) DQwrite = 1; // return to DQ high if write 1

  __delay_us(60); // hold value for remainder of timeslot
  DQwrite = 1;
  __delay_us(1);
}

//////////////////////////////////////////////////////////////////////////////
// read_byte()
// Reads a byte from the one-wire bus.
//
unsigned char ow_read_byte(void) {
  unsigned char i;
  unsigned char value = 0;
  for (i = 0; i < 8; i++) {
    if (ow_read_bit()) value |= 0x01 << i; // reads byte in, one byte at a time and then
    // shifts it left
  }
  return(value);
}

//////////////////////////////////////////////////////////////////////////////
// write_byte()
// Writes a byte to the one-wire bus.
//
void ow_write_byte(unsigned char val) {
  unsigned char i;
  unsigned char temp;
  for (i = 0; i < 8; i++) { // writes byte, one bit at a time
    temp = val >> i; // shifts val right 'i' spaces
    temp &= 0x01; // copy that bit to temp
    ow_write_bit(temp); // write bit in temp into
  }
}

