#include "hw_profile.h"

#include "sim900.h"
#include <string.h>


const char* op_profile[] = {
	OP_APN,
	OP_LOGIN,
	OP_PASSWD,
	"GPRS",
};
int send_command(const char *cmd, const char *resp, uint16_t tos)
{
    char bufresp[128];
    uint16_t to = tos * 10;
    uint16_t lenresp = strlen(resp);
    uint16_t lencmd = strlen(cmd);
    uint16_t nlines = 0;

    if (lencmd > 1) {
        gsm_printf("%s\r", cmd);
    }
    while (true) {
        __delay_ms(100);
        if (--to == 0) return 0;
        nlines = gsm_lines();
        while (nlines--) {
            if (gsm_gets(bufresp, 128)) {
                if (strlen(bufresp) > 0) {
                    dbg_gsm_printf("%u: %s\n", nlines, bufresp);
                    if (strncmp(resp, bufresp, lenresp) == 0) return 1;
                }
            }
        }
    }
}

int search_net(void)
{
    uint16_t i;
    int ret = 0;
    for (i = 0; i < TO_NET_SEARCH; i++) {
        if (send_command("AT+CGREG?", "+CGREG: 0,1", 1)) {
            ret = 1;
            break;
        }
        __delay_ms(500);
    }
    return ret;
}

int search_sms_net(void)
{
    uint16_t i;
    int ret = 0;
    for (i = 0; i < TO_NET_SEARCH; i++) {
        if (send_command("AT+CREG?", "+CREG: 0,1", 1)) {
            ret = 1;
            break;
        }
        __delay_ms(1000);
    }
    return ret;
}

int start_gprs(void)
{
    int ret = 0;
    uint16_t i = 0;
    char cmd[64];

    const int nGprsCmd = 4;
    char *gprs_cmd[] = {
        "AT+SAPBR=3,1,\"APN\",\"%s\"",
        "AT+SAPBR=3,1,\"USER\",\"%s\"",
        "AT+SAPBR=3,1,\"PWD\",\"%s\"",
        "AT+SAPBR=3,1,\"CONTYPE\",\"%s\"",
    };

    gsm_clear();
    for (i = 0; i < nGprsCmd; i++) {
        xsprintf(cmd, gprs_cmd[i], op_profile[i]);
        send_command(cmd, "OK", 1);
    }

    if (send_command("AT+SAPBR=1,1", "OK", TO_GPRS_CONNECT)) {
        send_command("AT+SAPBR=2,1", "OK", 1);
        ret = 1;
    } else {
        ret = 0;
    }
    return ret;
}

void stop_gprs(void)
{
    gsm_clear();
    send_command("AT+SAPBR=0,1", "OK", 1);
}

int open_ftp_session(const char *filename)
{
    uint16_t i = 0;
    int ret = 0;
    const int nFtpCmd = 7;
    char setfilename[32];
    char *ftp_cmd[] = {
        "AT+FTPSERV=\"78.108.80.36\"",
        "AT+FTPUN=\"f20256_1\"",
        "AT+FTPPW=\"qazQAZ\"",
        "AT+FTPTIMEOUT=5",
        "AT+FTPPUTPATH=\"/wind/1/\"",
        "AT+FTPPUTOPT=\"APPE\"",
        "AT+FTPTYPE=\"A\"",
    };
    gsm_clear();
    for (i = 0; i < nFtpCmd; i++) {
        send_command(ftp_cmd[i], "OK", 1);
    }
    xsprintf(setfilename, "AT+FTPPUTNAME=\"%s\"", filename);
    send_command(setfilename, "OK", 1);

    if (send_command("AT+FTPPUT=1", "+FTPPUT:1,1,", TO_FTP_OPEN)) {
        ret = 1;
    } else {
        ret = 0;
    }

    return ret;
}

int close_ftp_session(void)
{
    return send_command("AT+FTPPUT=2,0", "+FTPPUT:1,", 20);
}

int send_ftp_packet(const char *pbuff, int psize)
{
    int i, ret = 1;
    const char *ptr = pbuff;
    char tmpbuff[32], cmdbuff[32];

    xsprintf(tmpbuff, "+FTPPUT:2,%d", psize);
    xsprintf(cmdbuff, "AT+FTPPUT=2,%d", psize);

    if (send_command(cmdbuff, tmpbuff, 20)) {
        for (i = 0; i < psize; ++i) {
            gsm_putc(ptr[i]);
        }
        if (send_command("AT+FTPPUT=1", "+FTPPUT:1,1,", TO_FTP_OPEN)) {
            ret = 0;
        } else {
            ret = 1;
        }
    } else {
        ret = 1;
    }
    return ret;
}

int prepare_get_req(void)
{
    gsm_clear();
    if (!send_command("AT+HTTPINIT", "OK", 5)) return 1;
    if (!send_command("AT+HTTPPARA=\"CID\",1", "OK", 5)) return 1;
    return 0;
}

int send_get_line(const char* prefix, const char* line, char *answer)
{
    char urlbuf[128];
    char bufresp[128];
    uint16_t nlines = 0;

    xsprintf(urlbuf, "AT+HTTPPARA=\"URL\",\"%s%s\"", prefix, line);

    gsm_clear();
    if (!send_command("AT+HTTPINIT", "OK", 5)) return 1;
    if (!send_command("AT+HTTPPARA=\"CID\",1", "OK", 5)) return 1;
    if (!send_command(urlbuf, "OK", 5)) return 1;
    if (!send_command("AT+HTTPACTION=0", "OK", 5)) return 1;
    if (!wait_msg("+HTTPACTION:0", 30)) return 1;
    if (!send_command("AT+HTTPREAD", "+HTTPREAD", 30)) return 1;
    __delay_ms(1000);
    nlines = gsm_lines();
    if (nlines) {
        if (gsm_gets(bufresp, 128)) {
            if (strlen(bufresp) > 0) {
                dbg_gsm_printf("%u: %s\n", nlines, bufresp);
//                strcpy(answer, bufresp);
            }
        }
    }

    send_command("AT+HTTPTERM", "OK", 5);

    return 0;
}
