/************************************************************************************************
 *
 *  File:           buttons.h
 *
 *  Compilers:      HT-PICC STD
 *                  HT-PICC18 STD
 *
 *  Programmer:     Timofeev Victor
 *                  osa@pic24.ru, testerplus@mail.ru
 *
 *  Description:    Header for Task_Buttons
 *
 ************************************************************************************************/

#ifndef _TASK_BUTTONS_H
#define _TASK_BUTTONS_H

#include "buttonsconf.h"

//******************************************************************************
//
//      Types
//
//******************************************************************************

//------------------------------------------------------------------------------
//  Buttons type definition
//------------------------------------------------------------------------------

typedef struct {
  unsigned char   ucPressed;          // Current buttons state
  unsigned char   ucJitter    : 2;    // Counter for debouncing
  unsigned char   ucCounter   : 6;    // Counter forchecking hold buttons

  //------------------------------------------------------------------------------
#ifdef  BUTTONS_ENABLE_SHORT_HOLD

  unsigned char   ucHold;             // Hold buttons.
  unsigned char   ucShort;            // Event (must be cleared in software):
  // After releasing set to "1" if
  // there was not holding.
#endif
  //------------------------------------------------------------------------------
#ifdef  BUTTONS_ENABLE_JUST_PRESSED

  unsigned char   ucJustPressed;      // Event (must be cleared in software):
  // button was just pressed
#endif
  //------------------------------------------------------------------------------
#ifdef  BUTTONS_ENABLE_JUST_RELEASED

  unsigned char   ucJustReleased;     // Event (must be cleared in software):
  // button was just released
#endif
  //------------------------------------------------------------------------------
#ifdef  BUTTONS_ENABLE_CHANGED

  unsigned char   ucChanged;          // Event (must be cleared in software):
  // button state was just changed
  // (pressed, released or hold)
#endif
  //------------------------------------------------------------------------------

} TBUTTONS;

//******************************************************************************
//
//      Variables
//
//******************************************************************************

extern volatile TBUTTONS    m_Buttons;

//******************************************************************************
//
//      Function prototypes
//
//******************************************************************************

extern void ProcessButtons (void);




#endif

//******************************************************************************
//
//      THE END (buttons.h)
//
//******************************************************************************







