#ifndef LCD_I2C_H
#define LCD_I2C_H

/*****************************************************************************
 *
 * LCD
 *
 *****************************************************************************/

#include "lcd_i2c_conf.h"

#define LCDADDR		0b01111000
#define LCDCMD		0b10000000
#define LCDDATA		0b01000000

extern unsigned char LCD1Contrast, LCD2Contrast, LCD3Contrast;
extern unsigned char LCD1Contrast_low;
extern unsigned char LCD1Contrast_med;
extern unsigned char LCD1Contrast_high;


typedef struct tagLCDFlag {
    unsigned LCDInv :
            1;
    unsigned LCDAutoPartRefresh;
    unsigned ADRefresh :
            1;
    unsigned Button1Pressed;
    unsigned Button2Pressed;
    unsigned Button3Pressed;
    unsigned Button4Pressed;
    unsigned CycleFlag :
            1;
    unsigned MenuRefreshFlag :
            1;
    unsigned MenuBarFlag :
            1;
    unsigned ClockRefresh :
            1;
    unsigned White :
            1;
    unsigned DemoMode :
            1;
} LCDFLAGS;

extern volatile LCDFLAGS LCDFlags;

#define  _LCDAutoPartRefresh	LCDFlags.LCDAutoPartRefresh
#define  _LCDInv	 	LCDFlags.LCDInv
#define  _ADRefresh	 	LCDFlags.ADRefresh
#define	 _CycleFlag		LCDFlags.CycleFlag
#define	 _Button1Pressed	LCDFlags.Button1Pressed
#define	 _Button2Pressed	LCDFlags.Button2Pressed
#define	 _Button3Pressed	LCDFlags.Button3Pressed
#define	 _Button4Pressed	LCDFlags.Button4Pressed
#define	 _MenuRefreshFlag	LCDFlags.MenuRefreshFlag
#define	 _MenuBarFlag		LCDFlags.MenuBarFlag
#define  _ClockRefresh 		LCDFlags.ClockRefresh
#define  _White 			LCDFlags.White
#define	 _DemoMode			LCDFlags.DemoMode

typedef struct tagLCDRefreshFlags {

    union {

        struct {
            unsigned LCD1RefreshFlags :
                    4;
            unsigned LCD2RefreshFlags :
                    8;
            unsigned LCD3RefreshFlags :
                    4;
        };

        struct {
            unsigned LCD1Refresh3 :
                    1;
            unsigned LCD1Refresh2 :
                    1;
            unsigned LCD1Refresh1 :
                    1;
            unsigned LCD1Refresh0 :
                    1;
            unsigned LCD2Refresh7 :
                    1;
            unsigned LCD2Refresh6 :
                    1;
            unsigned LCD2Refresh5 :
                    1;
            unsigned LCD2Refresh4 :
                    1;
            unsigned LCD2Refresh3 :
                    1;
            unsigned LCD2Refresh2 :
                    1;
            unsigned LCD2Refresh1 :
                    1;
            unsigned LCD2Refresh0 :
                    1;
            unsigned LCD3Refresh3 :
                    1;
            unsigned LCD3Refresh2 :
                    1;
            unsigned LCD3Refresh1 :
                    1;
            unsigned LCD3Refresh0 :
                    1;
        };
    };


} LCDREFRESHFLAGS;

extern volatile LCDREFRESHFLAGS LCDRefreshFlags;

#define _LCD1RefreshFlags LCDRefreshFlags.LCD1RefreshFlags
#define _LCD2RefreshFlags LCDRefreshFlags.LCD2RefreshFlags
#define _LCD3RefreshFlags LCDRefreshFlags.LCD3RefreshFlags


#define _LCD1Refresh0 LCDRefreshFlags.LCD1Refresh0
#define _LCD1Refresh1 LCDRefreshFlags.LCD1Refresh1
#define _LCD1Refresh2 LCDRefreshFlags.LCD1Refresh2
#define _LCD1Refresh3 LCDRefreshFlags.LCD1Refresh3
#define _LCD2Refresh0 LCDRefreshFlags.LCD2Refresh0
#define _LCD2Refresh1 LCDRefreshFlags.LCD2Refresh1
#define _LCD2Refresh2 LCDRefreshFlags.LCD2Refresh2
#define _LCD2Refresh3 LCDRefreshFlags.LCD2Refresh3
#define _LCD2Refresh4 LCDRefreshFlags.LCD2Refresh4
#define _LCD2Refresh5 LCDRefreshFlags.LCD2Refresh5
#define _LCD2Refresh6 LCDRefreshFlags.LCD2Refresh6
#define _LCD2Refresh7 LCDRefreshFlags.LCD2Refresh7
#define _LCD3Refresh0 LCDRefreshFlags.LCD3Refresh0
#define _LCD3Refresh1 LCDRefreshFlags.LCD3Refresh1
#define _LCD3Refresh2 LCDRefreshFlags.LCD3Refresh2
#define _LCD3Refresh3 LCDRefreshFlags.LCD3Refresh3


#define mLCD1ClearDXDY(dx, dy)      LCD1ClearXY(LCD1CurX, LCD1CurY, dx, dy);

extern void I2C1Command(unsigned char);
extern void LCD1Init(unsigned char contrast);
extern void LCD1Clear(void);
extern void LCD1ClearXY(unsigned char, unsigned char, unsigned char, unsigned char);
extern void LCD1Refresh(void);
extern void LCD1PartialRefresh(void);
extern void LCD1SetContrast(unsigned char Value);


/*********************************************************************
 * EOF
 ********************************************************************/
#endif